package at.fhj.swd.dailyhelper.ui.activity;

import android.Manifest;
import android.app.ProgressDialog;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.pm.PackageManager;
import android.location.Location;
import android.os.Bundle;
import android.preference.PreferenceManager;
import android.support.annotation.NonNull;
import android.support.v4.app.ActivityCompat;
import android.support.v4.view.GravityCompat;
import android.support.v7.app.ActionBarDrawerToggle;
import android.view.MenuItem;
import android.support.design.widget.NavigationView;
import android.support.v4.widget.DrawerLayout;

import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.Toolbar;
import android.view.Menu;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.util.ArrayList;
import java.util.List;

import at.fhj.swd.dailyhelper.R;
import at.fhj.swd.dailyhelper.db.room.WeatherDB;
import at.fhj.swd.dailyhelper.model.Weather;
import at.fhj.swd.dailyhelper.network.HttpsGetTask;
import at.fhj.swd.dailyhelper.network.RequestCallback;
import at.fhj.swd.dailyhelper.service.LocationService;
import at.fhj.swd.dailyhelper.ui.fragemnt.HomeFragment;
import at.fhj.swd.dailyhelper.ui.fragemnt.MapFragment;
import at.fhj.swd.dailyhelper.ui.fragemnt.WeatherFragment;

public class MainActivity extends AppCompatActivity
        implements NavigationView.OnNavigationItemSelectedListener, RequestCallback {

    private DrawerLayout drawer;
    private ProgressDialog progressDialog;
    private SharedPreferences preferences;
    private WeatherFragment weatherFragment;
    private HomeFragment homeFragment;
    private WeatherDB database;
    private NavigationView navigationView;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        Toolbar toolbar = findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);

        weatherFragment = new WeatherFragment();
        homeFragment = new HomeFragment();

        progressDialog = new ProgressDialog(this);
        preferences = PreferenceManager.getDefaultSharedPreferences(this);

        database = WeatherDB.getDatabase(this);


        drawer = findViewById(R.id.drawer_layout);
        navigationView = findViewById(R.id.nav_view);
        navigationView.setNavigationItemSelectedListener(this);

        ActionBarDrawerToggle toggle = new ActionBarDrawerToggle(
                this, drawer, toolbar, R.string.navigation_drawer_open, R.string.navigation_drawer_close);
        drawer.addDrawerListener(toggle);
        toggle.syncState();

        if(savedInstanceState == null) {
            getSupportFragmentManager().beginTransaction().replace(R.id.fragment_container,homeFragment).commit();
            navigationView.setCheckedItem(R.id.nav_home);
        }

        startLocationService();
        fetchWeather();


    }

    @Override
    public void onBackPressed() {
        DrawerLayout drawer = findViewById(R.id.drawer_layout);
        if (drawer.isDrawerOpen(GravityCompat.START)) {
            drawer.closeDrawer(GravityCompat.START);
        } else {
            super.onBackPressed();
        }
    }

    private void startLocationService(){

        if (ActivityCompat.checkSelfPermission(this, Manifest.permission.ACCESS_FINE_LOCATION) == PackageManager.PERMISSION_GRANTED) {
            Intent intent = new Intent(this, LocationService.class);
            startService(intent);
        } else {
            ActivityCompat.requestPermissions(this, new String[]{Manifest.permission.ACCESS_FINE_LOCATION}, 12435 );
        }
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        if(requestCode == 12435){
            if(grantResults[0] == PackageManager.PERMISSION_GRANTED){
                startLocationService();
                fetchWeather();
                updateFragments();
            }
        }
    }

    private void stopLocationService(){
        Intent intent = new Intent(this, LocationService.class);
        stopService(intent);
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.main, menu);
        return true;
    }


    @Override
    public boolean onNavigationItemSelected(MenuItem item) {

        switch (item.getItemId()){
            case R.id.nav_home:
                getSupportFragmentManager().beginTransaction().replace(R.id.fragment_container, homeFragment).commit();
                break;
            case R.id.nav_map:
                getSupportFragmentManager().beginTransaction().replace(R.id.fragment_container, new MapFragment()).commit();
                break;
            case R.id.nav_weather:
                getSupportFragmentManager().beginTransaction().replace(R.id.fragment_container, weatherFragment).commit();
                fetchWeather();
                break;
        }

        drawer.closeDrawer(GravityCompat.START);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()){

            case R.id.action_service_on:
                startLocationService();
                return true;

            case R.id.action_service_off:
                stopLocationService();
                return true;

            case R.id.action_refresh:
                fetchWeather();
                return true;


            default:
                return false;
        }

    }

    private void updateFragments() {
        runOnUiThread(() -> {
            weatherFragment.refresh();
            homeFragment.refresh();

        });

    }

    private void dismissProgressDialog() {
        runOnUiThread(()-> progressDialog.dismiss());
    }


    public void fetchWeather() {
        HttpsGetTask httpsGetTask = new HttpsGetTask(this);
        SharedPreferences preferences = PreferenceManager.getDefaultSharedPreferences(this);
        float latitude = preferences.getFloat("latitude", -1000);
        float longitude = preferences.getFloat("longitude", -1000);

        if (latitude == -1000 || longitude == -1000) {
            httpsGetTask.execute("https://api.openweathermap.org/data/2.5/forecast?q=Vienna&limit=10&appid=030cadd0e9d6bdb9ae54ff0f841258ea&lang=de&units=metric");
        } else {
            httpsGetTask.execute("https://api.openweathermap.org/data/2.5/forecast?lat="+ latitude +"&lon=" + longitude + "&appid=030cadd0e9d6bdb9ae54ff0f841258ea&lang=de&units=metric");
        }
    }


    @Override
    public void onRequestStart() {
        progressDialog.setMessage("Fetching weather data...");
        progressDialog.show();
    }

    @Override
    public void onResult(String result) {
        new Thread(()->{

            List<Weather> weather;
            //parse result
            if(result == null){
                System.out.println("no weather");
            }else {
                try {
                    weather = parseWeather(result);
                    database.weatherDAO().clear();
                    database.weatherDAO().addAll(weather);
                    updateFragments();
                }catch (JSONException e){
                    e.printStackTrace();
                }
            }

            //notify fragments
            dismissProgressDialog();

        }).start();


    }

    private List<Weather> parseWeather(String json) throws JSONException {

        List<Weather> weather = new ArrayList<>();
        JSONObject jsonObject = new JSONObject(json);
        JSONArray results = jsonObject.getJSONArray("list");


        for (int i = 0; i < results.length(); i++){
            JSONObject weatherResult = results.getJSONObject(i);
            String date = weatherResult.getString("dt_txt");

            JSONArray weatherMetadata = weatherResult.getJSONArray("weather");
            JSONObject weatherMetadata0 = weatherMetadata.getJSONObject(0);

            String description = weatherMetadata0.getString("description");
            String icon = weatherMetadata0.getString("icon");

            JSONObject main = weatherResult.getJSONObject("main");

            double temp = main.getDouble("temp");
            double humidity = main.getDouble("humidity");

            Weather weatherData = new Weather(date, description, icon, temp, humidity);
            weather.add(weatherData);
        }
        return weather;


    }
}
